def picard_CalculateHsMetrics_to_df(stats_path, logger):
    select = 'BAIT_SET'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def do_picard_metrics(uuid, stats_path, input_state, bam, fasta, engine, logger, metrics_type, wxs_dict = None, vcf = None):
    if metrics_type == 'CollectJumpingLibraryMetrics':
        pass
    elif metrics_type == 'CollectVariantCallingMetrics':
        pass
    elif metrics_type == 'CollectWgsMetricsFromQuerySorted':
        pass
    elif metrics_type == 'CollectWgsMetricsFromSampledSites':
        pass
    elif metrics_type == 'CollectWgsMetricsWithNonZeroCoverage':
        pass
    elif metrics_type == 'EstimateLibraryComplexity':
        pass
    elif metrics_type == 'CalculateHsMetrics':
        table_name = 'picard_' + metrics_type
        df = picard_CalculateHsMetrics_to_df(stats_path, logger)
        if df is not None:
            df_list.append(df)
            table_name_list.append(table_name)
    else:
        logger.debug('Unknown metrics_type: %s' % metrics_type)
    for i, df in enumerate(df_list):
        logger.info('df_list enumerate i=%s:' % i)
        df['uuid'] = uuid
        df['bam'] = bam
        df['input_state'] = input_state
        df['fasta'] = fasta
        if vcf is not None:
            df['vcf'] = vcf
        table_name = table_name_list[i]
        df.to_sql(table_name, engine, if_exists='append')
    return



import sys
from collections import defaultdict

import pandas as pd

def run(uuid, stats_path, bam, input_state, engine, logger):
    val_error_dict = defaultdict(dict)
    with open(stats_path, 'r') as f_open:
        for line in f_open:
            if line.startswith('ERROR:'):
                validation_type = 'ERROR'
                line_split = line.strip().split(',')
                if len(line_split) == 4:
                    line_error = line_split[2:]
                    line_error = ', '.join(line_error)
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                elif len(line_split) == 3:
                    line_error = line_split[2]
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                elif len(line_split) == 2:
                    line_error = line_split[1]
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                elif len(line_split) == 1:
                    line_error = line_split[0]
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                else:
                    logger.debug('validation_type=ERROR')
                    logger.debug('line: %s' % line)
                    logger.debug('Need to handle this comma amount: %s' % len(line_split))
                    sys.exit(1)
            elif line.startswith('WARNING:'):
                validation_type = 'WARNING'
                line_split = line.strip().split(',')
                if len(line_split) == 4:
                    line_error = line_split[2:]
                    line_error = ', '.join(line_error)
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                elif len(line_split) == 3:
                    line_error = line_split[2]
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                elif len(line_split) == 2:
                    line_error = line_split[1].strip()
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                elif len(line_split) == 1:
                    line_error = line_split[0]
                    if line_error in val_error_dict[validation_type].keys():
                        val_error_dict[validation_type][line_error] += 1
                    else:
                        val_error_dict[validation_type][line_error] = 1
                else:
                    logger.debug('validation_type=WARNING')
                    logger.debug('line: %s' % line)
                    logger.debug('Need to handle this comma amount: %s' % len(line_split))
                    sys.exit(1)
            elif line.startswith('No errors found'):
                validation_type = 'PASS'
                line_error = line.strip()
                val_error_dict[validation_type][line_error] = 1
            else:
                logger.info('unknown picard validation line')
                logger.info('line=%s' % line)
                sys.exit(1)

    validation_type = 'ERROR'
    for akey in sorted(val_error_dict[validation_type].keys()):
        store_dict = dict()
        store_dict['value'] = akey
        store_dict['count'] = val_error_dict[validation_type][akey]
        store_dict['uuid'] = [uuid]  # a non scalar
        store_dict['bam'] = bam
        store_dict['severity'] = validation_type
        logger.info('store_validate_error() store_dict=%s' % store_dict)
        df = pd.DataFrame(store_dict)
        table_name = 'picard_ValidateSamFile'
        df.to_sql(table_name, engine, if_exists='append')
    validation_type = 'WARNING'
    for akey in sorted(val_error_dict[validation_type].keys()):
        store_dict = dict()
        store_dict['value'] = akey
        store_dict['count'] = val_error_dict[validation_type][akey]
        store_dict['uuid'] = [uuid]  # a non scalar
        store_dict['bam'] = bam
        store_dict['severity'] = validation_type
        logger.info('store_validate_error() store_dict=%s' % store_dict)
        df = pd.DataFrame(store_dict)
        table_name = 'picard_ValidateSamFile'
        df.to_sql(table_name, engine, if_exists='append')
    validation_type = 'PASS'
    for akey in sorted(val_error_dict[validation_type].keys()):
        store_dict = dict()
        store_dict['value'] = akey
        store_dict['count'] = val_error_dict[validation_type][akey]
        store_dict['uuid'] = [uuid]  # a non scalar
        store_dict['bam'] = bam
        store_dict['severity'] = validation_type
        logger.info('store_validate_error() store_dict=%s' % store_dict)
        df = pd.DataFrame(store_dict)
        table_name = 'picard_ValidateSamFile'
        df.to_sql(table_name, engine, if_exists='append')
    return



#!/usr/bin/env python

import argparse
import logging
import os
import sys

import sqlalchemy

from .metrics import picard_collectalignmentsummarymetrics
from .metrics import picard_collectmultiplemetrics
from .metrics import picard_collectoxogmetrics
from .metrics import picard_collectwgsmetrics
from .metrics import picard_markduplicates
from .metrics import picard_validatesamfile
#from metrics.picard_calculatehsmetrics_gdc import picard_calculatehsmetrics as picard_calculatehsmetrics_gdc
# from metrics.picard_calculatehsmetrics_tcga import picard_calculatehsmetrics as picard_calculatehsmetrics_tcga
# from metrics.picard_calculatehsmetrics_target import picard_calculatehsmetrics as picard_calculatehsmetrics_target

def get_param(args, param_name):
    if vars(args)[param_name] == None:
        sys.exit('--'+ param_name + ' is required')
    else:
        return vars(args)[param_name]
    return
    
def setup_logging(tool_name, args, uuid):
    logging.basicConfig(
        filename=os.path.join(uuid + '_' + tool_name + '.log'),
        level=args.level,
        filemode='w',
        format='%(asctime)s %(levelname)s %(message)s',
        datefmt='%Y-%m-%d_%H:%M:%S_%Z',
    )
    logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
    logger = logging.getLogger(__name__)
    return logger

def main():
    parser = argparse.ArgumentParser('picard docker tool')

    # Logging flags.
    parser.add_argument('-d', '--debug',
        action = 'store_const',
        const = logging.DEBUG,
        dest = 'level',
        help = 'Enable debug logging.',
    )
    parser.set_defaults(level = logging.INFO)
    
    # Required flags.
    parser.add_argument('--input_state',
                        required = True
    )
    parser.add_argument('--metric_name',
                        required = True,
                        help = 'picard tool'
    )
    parser.add_argument('--metric_path',
                        required = True
    )
    parser.add_argument('--uuid',
                        required = True,
                        help = 'uuid string',
    )
    
    # Tool flags
    parser.add_argument('--bam_library_kit_json_path',
                        required = False
    )
    parser.add_argument('--bam',
                        required = False
    )
    parser.add_argument('--vcf',
                        required = False
    )
    parser.add_argument('--readgroup_json_path',
                        required = False
    )
    parser.add_argument('--fasta',
                        required = False
    )

    
    # setup required parameters
    args = parser.parse_args()
    input_state = args.input_state
    metric_name = args.metric_name
    metric_path = args.metric_path
    uuid = args.uuid

    logger = setup_logging('picard_' + metric_name, args, uuid)

    sqlite_name = uuid + '.db'
    engine_path = 'sqlite:///' + sqlite_name
    engine = sqlalchemy.create_engine(engine_path, isolation_level='SERIALIZABLE')


    # elif metric_name == 'CollectHsMetrics_target':
    #     bam = get_param(args, 'bam')
    #     fasta = get_param(args, 'fasta')
    #     json_path = get_param(args, 'json_path')
    #     interval_dir = get_param(args, 'interval_dir')
    #     wxs_dict['bait_intervals_path'] = bait_intervals_path
    #     wxs_dict['target_intervals_path'] = target_intervals_path
    #     picard_calculatehsmetrics_target.run(uuid, bam, input_state, json_path, interval_dir, engine, logger, wxs_dict = wxs_dict)
    # elif metric_name == 'CollectHsMetrics_tcga':
    #     bam = get_param(args, 'bam')
    #     fasta = get_param(args, 'fasta')
    #     json_path = get_param(args, 'json_path')
    #     interval_dir = get_param(args, 'interval_dir')
    #     wxs_dict['bait_intervals_path'] = bait_intervals_path
    #     wxs_dict['target_intervals_path'] = target_intervals_path
    #     picard_calculatehsmetrics_tcga.run(uuid, bam, input_state, json_path, interval_dir, engine, logger, wxs_dict = wxs_dict)
    # if metric_name == 'CollectHsMetrics_gdc':
    #     bam = get_param(args, 'bam')
    #     bam_library_kit_json_path = get_param(args, 'bam_library_kit_json_path')
    #     input_state = get_param(args, 'input_state')
    #     orig_bam_name = get_param(args, 'outbam_name')
    #     fasta = get_param(args, 'fasta')
    #     readgroup_json_path = get_param(args, 'readgroup_json_path')
    #     picard_calculatehsmetrics_gdc.run(uuid, bam, readgroup_json_path, bam_library_kit_json_path, orig_bam_name, input_state, engine, logger)
    if metric_name == 'CollectAlignmentSummaryMetrics':
        bam = get_param(args, 'bam')
        fasta = get_param(args, 'fasta')
        picard_collectalignmentsummarymetrics.run(uuid, metric_path, bam, fasta, input_state, engine, logger, metric_name)
    elif metric_name == 'CollectMultipleMetrics':
        bam = get_param(args, 'bam')
        fasta = get_param(args, 'fasta')
        input_state = get_param(args, 'input_state')
        vcf = get_param(args, 'vcf')
        picard_collectmultiplemetrics.run(uuid, metric_path, bam, fasta, vcf, input_state, engine, logger)
    elif metric_name == 'CollectOxoGMetrics':
        bam = get_param(args, 'bam')
        fasta = get_param(args, 'fasta')
        input_state = get_param(args, 'input_state')
        vcf = get_param(args, 'vcf')
        picard_collectoxogmetrics.run(uuid, metric_path, bam, fasta, vcf, input_state, engine, logger, metric_name)
    elif metric_name == 'CollectWgsMetrics':
        bam = get_param(args, 'bam')
        fasta = get_param(args, 'fasta')
        input_state = get_param(args, 'input_state')
        picard_collectwgsmetrics.run(uuid, metric_path, bam, fasta, input_state, engine, logger, metric_name)
    elif metric_name == 'MarkDuplicates':
        bam = get_param(args, 'bam')
        input_state = get_param(args, 'input_state')
        picard_markduplicates.run(uuid, metric_path, bam, input_state, engine, logger, metric_name)
    # elif metric_name == 'MarkDuplicatesWithMateCigar':
    #     bam = get_param(args, 'bam')
    #     input_state = get_param(args, 'input_state')
    #     picard_markduplicateswithmatecigar.run(uuid, bam, input_state, engine, logger)
    elif metric_name == 'ValidateSamFile':
        bam = get_param(args, 'bam')
        input_state = get_param(args, 'input_state')
        picard_validatesamfile.run(uuid, metric_path, bam, input_state, engine, logger)
    else:
        sys.exit('No recognized tool was selected')
        
    return


if __name__ == '__main__':
    main()



import pandas as pd

def get_data_dict(stats_path, logger):
    data_dict = dict()
    read_header = False
    
    with open(stats_path, 'r') as f_open:
        for line in f_open:
            if line.startswith("## HISTOGRAM"):
                break
            if line.startswith('#') or len(line) < 5:
                continue
            if not read_header:
                value_key_list = line.strip('\n').split('\t')
                logger.info('picard_markduplicates_to_dict() header value_key_list=\n\t%s' % value_key_list)
                logger.info('len(value_key_list=%s' % len(value_key_list))
                read_header = True
            else:
                data_list = line.strip('\n').split('\t')
                logger.info('picard_markduplicates_do_dict() data_list=\n\t%s' % data_list)
                logger.info('len(data_list)=%s' % len(data_list))
                for value_pos, value_key in enumerate(value_key_list):
                    data_dict[value_key] = data_list[value_pos]
    return data_dict

def run(uuid, stats_path, bam, input_state, engine, logger, metric_name):
    data_dict = get_data_dict(stats_path, logger)
    data_dict['uuid'] = [uuid]
    data_dict['bam'] = bam
    data_dict['input_state'] = input_state
    df = pd.DataFrame(data_dict)
    table_name = 'picard_' + metric_name
    df.to_sql(table_name, engine, if_exists='append')
    return



import json
import os
import sys

import pandas as pd

def get_key_interval_dicts_from_json(key_intervalname_json_path, logger):
    with open(key_intervalname_json_path, 'r') as json_path_open:
        json_data = json.load(json_path_open)
    return json_data


def all_tsv_to_df(tsv_path, logger):
    logger.info('all_tsv_to_df open: %s' % tsv_path)
    data_dict = dict()
    with open(tsv_path, 'r') as tsv_open:
        i = 0
        for line in tsv_open:
            line = line.strip('\n')
            line_split = line.split('\t')
            data_dict[i] = line_split
            i += 1
    logger.info('data_dict=\n%s' % data_dict)
    df = pd.DataFrame.from_dict(data_dict, orient='index')
    logger.info('df=\n%s' % df)
    return df


def picard_select_tsv_to_df(stats_path, select, logger):
    read_header = False
    data_dict = dict()
    if not os.path.exists(stats_path):
        logger.info('the stats file %s do not exist, so return None' % stats_path)
        return None
    logger.info('stats_path=%s' % stats_path)
    with open(stats_path, 'r') as stats_open:
        i = 0
        for line in stats_open:
            line = line.strip('\n')
            if line.startswith('#'):
                continue
            line_split = line.split('\t')
            if not read_header and len(line_split) > 1:
                if select == line_split[0]:
                    header = line_split
                    read_header = True
            elif read_header and len(line_split) == 1:
                df_index = list(range(len(data_dict)))
                df = pd.DataFrame.from_dict(data_dict, orient='index')
                df.columns = header
                return df
            elif read_header and len(line_split) > 0:
                if len(line_split) == len(header):
                    data_dict[i] = line_split
                    i += 1
            elif not read_header and len(line_split) == 1:
                continue
            else:
                logger.info('strange line: %s' % line)
                sys.exit(1)
    if not read_header:
        logger.info('bam file was probably too small to generate stats as header not read: %s' % stats_path)
        return None
    logger.debug('no data saved to df')
    sys.exit(1)
    return None



import os

from .metrics_util import picard_select_tsv_to_df


def picard_CollectAlignmentSummaryMetrics_to_df(stats_path, logger):
    select = 'CATEGORY'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_CollectBaseDistributionByCycle_to_df(stats_path, logger):
    select = 'READ_END'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_CollectInsertSizeMetrics_metrics_to_df(stats_path, logger):
    select = 'MEDIAN_INSERT_SIZE'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_CollectInsertSizeMetrics_histogram_to_df(stats_path, logger):
    select = 'insert_size'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    if df is not None:
        keep_column_list = ['insert_size', 'All_Reads.fr_count', 'All_Reads.rf_count', 'All_Reads.tandem_count']
        drop_column_list = [ column for column in df.columns if column not in keep_column_list]
        needed_column_list = [ column for column in keep_column_list if column not in df.columns ]
        #drop readgroup specific columns as the bam is already one readgroup
        logger.info('pre drop df=\n%s' % df)
        df.drop(drop_column_list, axis=1, inplace=True)
        logger.info('post drop df=\n%s' % df)
        #add columns that could be present in other files
        for needed_column in needed_column_list:
            df[needed_column] = None
    return df


def picard_MeanQualityByCycle_to_df(stats_path, logger):
    select = 'CYCLE'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_CollectQualityYieldMetrics_to_df(stats_path, logger):
    select = 'TOTAL_READS'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_QualityScoreDistribution_to_df(stats_path, logger):
    select = 'QUALITY'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_CollectGcBiasMetrics_to_df(stats_path, logger):
    select = 'ACCUMULATION_LEVEL'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_CollectSequencingArtifactMetrics_to_df(stats_path, logger):
    select = 'SAMPLE_ALIAS'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def run(uuid, stats_path, bam, fasta, vcf, input_state, engine, logger):
    stats_dir = os.path.dirname(stats_path)
    stats_name = os.path.basename(stats_path)
    stats_base, stats_ext = os.path.splitext(stats_name)

    df_list = list()
    table_name_list = list()

    
    table_name = 'picard_CollectAlignmentSummaryMetrics'
    stats_file = stats_base + '.alignment_summary_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectAlignmentSummaryMetrics_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name_list.append(table_name)

    table_name = 'picard_CollectBaseDistributionByCycle'
    stats_file = stats_base + '.base_distribution_by_cycle_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectBaseDistributionByCycle_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name_list.append(table_name)

    table_name = 'picard_CollectInsertSizeMetric'
    stats_file = stats_base + '.insert_size_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectInsertSizeMetrics_metrics_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name += '_metrics'
        table_name_list.append(table_name)

    table_name = 'picard_CollectInsertSizeMetric'
    df = picard_CollectInsertSizeMetrics_histogram_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name += '_histogram'
        table_name_list.append(table_name)

    table_name = 'picard_MeanQualityByCycle'
    stats_file = stats_base + '.quality_by_cycle_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_MeanQualityByCycle_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name_list.append(table_name)

    table_name = 'picard_QualityScoreDistribution'
    stats_file = stats_base + '.quality_distribution_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_QualityScoreDistribution_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name_list.append(table_name)

    table_name = 'picard_CollectGcBiasMetrics'
    stats_file = stats_base + '.gc_bias.summary_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectGcBiasMetrics_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name += '_summary'
        table_name_list.append(table_name)

    table_name = 'picard_CollectGcBiasMetrics'
    stats_file = stats_base + '.gc_bias.detail_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectGcBiasMetrics_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name += '_detail'
        table_name_list.append(table_name)

    table_name = 'picard_CollectQualityYieldMetrics'
    stats_file = stats_base + '.quality_yield_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectQualityYieldMetrics_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name_list.append(table_name)

    table_name = 'picard_CollectSequencingArtifactMetrics'
    stats_file = stats_base + '.pre_adapter_detail_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectSequencingArtifactMetrics_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name += '_detail'
        table_name_list.append(table_name)

    table_name = 'picard_CollectSequencingArtifactMetrics'
    stats_file = stats_base + '.pre_adapter_summary_metrics'
    stats_path = os.path.join(stats_dir, stats_file)
    df = picard_CollectSequencingArtifactMetrics_to_df(stats_path, logger)
    if df is not None:
        df_list.append(df)
        table_name += '_summary'
        table_name_list.append(table_name)
            
    for i, df in enumerate(df_list):
        logger.info('df_list enumerate i=%s:' % i)
        df['uuid'] = uuid
        df['bam'] = bam
        df['input_state'] = input_state
        df['fasta'] = fasta
        if vcf is not None:
            df['vcf'] = vcf
        table_name = table_name_list[i]
        df.to_sql(table_name, engine, if_exists='append')
    return



import os

from .metrics_util import picard_select_tsv_to_df

def picard_CollectWgsMetrics_to_df(stats_path, logger):
    select = 'GENOME_TERRITORY'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df


def picard_CollectWgsMetrics_histogram_to_df(stats_path, logger):
    select = 'coverage'
    df = picard_select_tsv_to_df(stats_path, select, logger)
    return df

def run(uuid, stats_path, bam, fasta, input_state, engine, logger, metric_name):
    table_name = 'picard_' + metric_name
    df = picard_CollectWgsMetrics_to_df(stats_path, logger)
    if df is None:
        sys.exit('No metrics collected from: %s' % stats_path)
    else:
        df['uuid'] = uuid
        df['bam'] = bam
        df['input_state'] = input_state
        df['fasta'] = fasta
        df.to_sql(table_name, engine, if_exists='append')

    table_name += '_histogram'
    df = picard_CollectWgsMetrics_histogram_to_df(stats_path, logger)
    if df is None:
        sys.exit('No metrics collected from: %s' % stats_path)
    else:
        df['uuid'] = uuid
        df['bam'] = bam
        df['input_state'] = input_state
        df['fasta'] = fasta
        df.to_sql(table_name, engine, if_exists='append')
    return
